from django.db import models

class Watermark(models.Model):
    name = models.CharField(max_length=50)
    image = models.ImageField(upload_to='watermarks')
    is_active = models.BooleanField(default=True, blank=True)
    date_created = models.DateTimeField(auto_now_add=True)
    date_updated = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ['name']


"""
Utilities for applying a watermark to an image using PIL.

Original Source: http://code.activestate.com/recipes/362879/
"""
import Image, ImageEnhance
import random

def _percent(var):
    """
    Just a simple interface to the _val function with a more meaningful name.
    """
    return _val(var, True)

def _int(var):
    """
    Just a simple interface to the _val function with a more meaningful name.
    """
    return _val(var)

def _val(var, is_percent=False):
    """
    Tries to determine the appropriate value of a particular variable that is
    passed in.  If the value is supposed to be a percentage, a whole integer
    will be sought after and then turned into a floating point number between
    0 and 1.  If the value is supposed to be an integer, the variable is cast
    into an integer.
    """
    try:
        if is_percent:
            var = float(int(var.strip('%')) / 100.0)
        else:
            var = int(var)
    except ValueError:
        raise ValueError('invalid watermark parameter: ' + var)
    return var

def reduce_opacity(img, opacity):
    """
    Returns an image with reduced opacity.
    """
    assert opacity >= 0 and opacity <= 1

    if img.mode != 'RGBA':
        img = img.convert('RGBA')
    else:
        img = img.copy()

    alpha = img.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    img.putalpha(alpha)

    return img

def determine_scale(scale, img, mark):
    """
    Scales an image using a specified ratio or 'F'.  If `scale` is 'F', the 
    image is scaled to be as big as possible to fit in `img` without falling off
    the edges.  Returns the scaled `mark`.
    """
    
    if scale:
        try:
            scale = float(scale)
        except ValueError:
            pass

        if (isinstance(scale, str) or isinstance(scale, unicode)) \
            and scale.lower() == 'f':
            # scale, but preserve the aspect ratio
            scale = min(
                        float(img.size[0]) / mark.size[0], 
                        float(img.size[1]) / mark.size[1]
                       )
        elif not isinstance(scale, float) and not isinstance(scale, int):
            raise ValueError('Invalid scale value "%s"!  Valid values are 1) "F" for ratio-preserving scaling and 2) floating-point numbers and integers greater than 0.' % scale)

        # determine the new width and height
        w = int(mark.size[0] * scale)
        h = int(mark.size[1] * scale)

        # apply the new width and height, and return the new `mark`
        return (w, h)
    else:
        return mark.size

def determine_rotation(rotation, mark):
    """
    Determines the number of degrees to rotate the watermark image.
    """
    
    if (isinstance(rotation, str) or isinstance(rotation, unicode)) \
        and rotation.lower() == 'r':
        rotation = random.randint(0, 359)
    else:
        rotation = _int(rotation)

    return rotation

def determine_position(position, img, mark):
    """
    Options:
        TL: top-left
        TR: top-right
        BR: bottom-right
        BL: bottom-left
        C: centered
        R: random
        X%xY%: relative positioning on both the X and Y axes
        X%xY: relative positioning on the X axis and absolute positioning on the 
              Y axis
        XxY%: absolute positioning on the X axis and relative positioning on the
              Y axis
        XxY: absolute positioning on both the X and Y axes
    """
    
    max_left = img.size[0] - mark.size[0]
    max_top = img.size[1] - mark.size[1]
    
    if not position:
        position = 'r'

    if isinstance(position, tuple):
        left, top = position
    elif isinstance(position, str) or isinstance(position, unicode):
        position = position.lower()
        
        # corner positioning
        if position in ['tl', 'tr', 'br', 'bl']:
            if 't' in position:
                top = 0
            elif 'b' in position:
                top = max_top
            if 'l' in position:
                left = 0
            elif 'r' in position:
                left = max_left

        # center positioning
        elif position == 'c':
            left = int(max_left / 2)
            top = int(max_top / 2)

        # random positioning
        elif position == 'r':
            left = random.randint(0, max_left)
            top = random.randint(0, max_top)

        # relative or absolute positioning
        elif 'x' in position:
            left, top = position.split('x')
            
            if '%' in left:
                left = max_left * _percent(left)
            else:
                left = _int(left)
            
            if '%' in top:
                top = max_top * _percent(top)
            else:
                top = _int(top)

    return (left, top)

def determine_parameter_values(img, mark, position=(0, 0), opacity=1, scale=1.0, tile=False, greyscale=False, rotation=0):
    """
    Examines the input parameters to determine what the actual values will be 
    for generating the watermark.
    """
    position = determine_position(position, img, mark)
    scale = determine_scale(scale, img, mark)
    rotation = determine_rotation(rotation, mark)
    
    return {
        'position': position, 
        'opacity':  opacity, 
        'scale':    scale, 
        'tile':     tile, 
        'greyscale': greyscale, 
        'rotation': rotation}

def watermark(img, mark, position=(0, 0), opacity=1, scale=1.0, tile=False, greyscale=False, rotation=0, return_name=False):
    """
    Adds a watermark to an image.
    """
    if opacity < 1:
        mark = reduce_opacity(mark, opacity)
    
    scale = determine_scale(scale, img, mark)
    mark = mark.resize(scale)

    if greyscale and mark.mode != 'LA':
        mark = mark.convert('LA')
    
    rotation = determine_rotation(rotation, mark)
    if rotation != 0:
        # give soem leeway for rotation overlapping
        new_w = mark.size[0] * 1.5
        new_h = mark.size[1] * 1.5
        
        new_mark = Image.new('RGBA', (new_w, new_h), (0,0,0,0))
        
        # center the watermark in the newly resized image
        new_l = (new_w - mark.size[0]) / 2
        new_t = (new_h - mark.size[1]) / 2
        new_mark.paste(mark, (new_l, new_t))
        
        mark = new_mark.rotate(rotation)
    
    position = determine_position(position, img, mark)

    if img.mode != 'RGBA':
        img = img.convert('RGBA')

    # make sure we have a tuple for a position now
    assert isinstance(position, tuple), 'Invalid position "%s"!' % position

    # create a transparent layer the size of the image and draw the
    # watermark in that layer.
    layer = Image.new('RGBA', img.size, (0,0,0,0))
    if tile:
        first_y = position[1] % mark.size[1] - mark.size[1]
        first_x = position[0] % mark.size[0] - mark.size[0]
        
        for y in range(first_y, img.size[1], mark.size[1]):
            for x in range(first_x, img.size[0], mark.size[0]):
                layer.paste(mark, (x, y))
    else:
        layer.paste(mark, position)

    # composite the watermark with the layer
    return Image.composite(layer, img, layer)

def test():
    im = Image.open('test.png')
    mark = Image.open('overlay.png')
    watermark(im, mark, 
                tile=True, 
                opacity=0.5, 
                rotation=30).save('test1.png')

    watermark(im, mark, 
                scale='F').save('test2.png')

    watermark(im, mark, 
                position=(100, 100), 
                opacity=0.5, 
                greyscale=True, 
                rotation=-45).save('test3.png')

    watermark(im, mark, 
                position='C',
                tile=False, 
                opacity=0.2,
                scale=2, 
                rotation=30).save('test4.png')

if __name__ == '__main__':
    test()



from django import template
from django.conf import settings
from watermarker.models import Watermark
from watermarker import utils
from datetime import datetime
import Image
import urlparse
import os
import random

register = template.Library()

# determine the quality of the image after the watermark is applied
try:
    QUALITY = settings.WATERMARKING_QUALITY
except AttributeError:
    QUALITY = 85

def _get_path_from_url(url, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """
    Makes a filesystem path from the specified URL
    """

    if url.startswith(url_root):
        url = url[len(url_root):] # strip media root url

    return os.path.normpath(os.path.join(root, url))

def watermark(url, args=''):
    """
    A filter to create a watermarked copy of an image.  The parameters are as
    follows::

    - [name]::
        This is the name of the Watermark object that you wish to apply to the
        image.

    - position::
        - XxY where X is either a specific pixel position on the x-axis or a
            percentage of the total width of the image and Y is a specific
            pixel position on the y-axis of the image or a percentage of the
            total height of the image.  If either X or Y is a percentage, you
            must use a percent sign.  This is not used if either one of the
            `tiled` or `scale` parameters are true.

            Examples::
                50%x50% - positions the watermark at the center of the image.
                50%x100 - positions the watermark at the midpoint of the total
                    width of the image and 100 pixels from the top of the image
                100x50% - positions the watermark at the midpoint of the total
                    height of the image and 100 pixels from the left edge of
                    the image
                100x100 - positions the top-left corner of the watermark at 100
                    pixels from the top of the image and 100 pixels from the
                    left edge of the image.
        - br, bl, tr, tl where `b` means 'bottom', `t` means 'top', `l` means
            'left', and `r` means 'right'.  This will position the watermark at
            the extreme edge of the original image with just enough room for
            the watermark to "fully show".  This assumes the watermark is not
            as big as the original image.
        - just R for random placement
        - just C to center the watermark
    - opacity::
        - X where X is an integer from 0 to 100.  This value represents the
            transparency level of the watermark when it is applied.
    - tile::
        - 0 or 1 to specify whether or not the watermark shall be tiled across
            the entire image.
    - scale::
        - a floating-point number above 0 to specify the scaling for the 
            watermark.  If you want the watermark to be scaled to its maximum
            without falling off the edge of the target image, use 'F'.  By 
            default, scale is set to 1.0, or 1:1 scaling.
    - greyscale::
        - 0 or 1 to specify whether or not the watermark should be converted to
            a greyscale image before applying it to the target image.
    - rotation::
        - 0 to 359 to specify the number of degrees to rotate the watermark
            before applying it to the target image.  Alternatively, you may
            set rotation=R for a random rotation value.
    """

    basedir = os.path.dirname(url) + '/'
    base, ext = os.path.splitext(os.path.basename(url))

    # initialize some variables
    args = args.split(',')
    name = args.pop(0)
    top = left = 0
    opacity = 0.5
    tile = False
    scale = 1.0
    greyscale = False
    rotation = 0
    position = None

    # look for the specified watermark by name.  If it's not there, go no further
    try:
        watermark = Watermark.objects.get(name=name, is_active=True)
    except Watermark.DoesNotExist:
        return url

    # iterate over all parameters to see what we need to do
    for arg in args:
        key, value = arg.split('=')
        if key == 'position':
            position = value
        elif key == 'opacity':
            opacity = _percent(value)
        elif key == 'tile':
            tile = bool(int(value))
        elif key == 'scale':
            scale = value
        elif key == 'greyscale':
            greyscale = bool(int(value))
        elif key == 'rotation':
            rotation = value

    # open the target image file along with the watermark image
    target_path = _get_path_from_url(url)
    target = Image.open(target_path)
    mark = Image.open(watermark.image.path)
    
    # determine the actual value that the parameters provided will render
    params = utils.determine_parameter_values(target, mark, position, opacity, scale, tile, greyscale, rotation)
    params.update({
                    'base': base,
                    'watermark': watermark.id,
                    'opacity': params['opacity'] * 100,
                    'left': params['position'][0],
                    'top': params['position'][1],
                    'ext': ext
                   })

    # come up with a good filename for this watermarked image
    wm_name = '%(base)s_wm_w%(watermark)i_o%(opacity)i_gs%(greyscale)i_r%(rotation)i'
    if params['position']:
        wm_name += '_p%(left)sx%(top)s'
    if params['scale'] and params['scale'] != mark.size:
        wm_name += '_s%i' % (float(params['scale'][0]) / mark.size[0] * 100)
    if params['tile']:
        wm_name += '_tiled'
    wm_name += '%(ext)s'

    # make thumbnail filename
    wm_name = wm_name % params

    # figure out where the watermark would be saved on the filesystem
    new_file = urlparse.urljoin(basedir, wm_name)
    new_path = _get_path_from_url(new_file)

    # see if the image already exists on the filesystem.  If it does, use it.
    if os.access(new_path, os.R_OK):
        # see if the Watermark object was modified since the file was created
        modified = datetime.fromtimestamp(os.path.getmtime(new_path))

        # only return the old file if things appear to be the same
        if modified >= watermark.date_updated:
            return new_file

    # create the watermarked image on the filesystem
    wm_image = utils.watermark(target, 
                               mark, 
                               position=position, 
                               opacity=opacity, 
                               scale=scale,
                               tile=tile,
                               greyscale=greyscale, 
                               rotation=rotation)
    wm_image.save(new_path, quality=QUALITY)

    # send back the URL to the new, watermarked image
    return urlparse.urljoin(basedir, wm_name)

register.filter(watermark)
